/*
 * Citadel/UX Intelligent Network Processor for IGnet/Open networks v3.6
 * designed and written by Art Cancro @ Uncensored Communications Group
 * see copyright.doc for copyright information
 */

/*
 * Specify where netproc should log to, and the mode for opening the file.
 * If you are logging to a file, NPLOGMODE should be a; if you are logging to
 * a device or fifo it should be w.
 */
#define NPLOGFILE "./netproc.log" 
#define NPLOGMODE "a"
#define EXPIRY_TIME (2592000L)		/* system purge time */
#define LOCKFILE "/usr/spool/uucp/LCK.netproc"

/* #define DEBUG 1 */


#include "sysdep.h"
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include "citadel.h"

/* A list of users you wish to filter out of incoming traffic can be kept
 * in ./network/filterlist -- messages from these users will be automatically
 * moved to FILTERROOM.  Normally this will be the same as TWITROOM (the
 * room problem user messages are moved to) but you can override this by
 * specifying a different room name here.
 */
#ifndef FILTERROOM
#define FILTERROOM TWITROOM
#endif

struct msglist {
	struct msglist *next;
	long m_num;
	long m_pos;
	int m_room;
	};

struct rmlist {
	struct rmlist *next;
	char rm_name[20];
	long rm_lastsent;
	};

struct filterlist {
	struct filterlist *next;
	char f_person[64];
	char f_room[64];
	char f_system[64];
	};

struct syslist {
	struct syslist *next;
	char s_name[16];
	char s_type[4];
	char s_nexthop[128];
	long s_lastcontact;
	char s_humannode[64];
	char s_phonenum[32];
	char s_gdom[64];
	};

struct minfo {
	char A[512];
	long I;
	char N[512];
	char O[512];
	char R[512];
	long T;
	char D[512];
	char C[512];
	char nexthop[32];
	char H[512];
	char S[512];
	char B[512];
	char G[512];
	};


void lock_msgmain();
void unlock_msgmain();
void attach_to_server();

struct filterlist *filter = NULL;
char roomnames[MAXROOMS][20];
char roomdirs[MAXROOMS][15];
struct syslist *slist = NULL;

void get_config();
struct config config;

int struncmp(lstr,rstr,len)
char lstr[],rstr[];
int len; {
	int pos = 0;
	char lc,rc;
	while (pos<len) {
		lc=tolower(lstr[pos]);
		rc=tolower(rstr[pos]);
		if ((lc==0)&&(rc==0)) return(0);
		if (lc<rc) return(-1);
		if (lc>rc) return(1);
		pos=pos+1;
		}
	return(0);
	}

/* redefine strucmp, just in case we're using an old version of citadel.h
 * that has it as a separate routine
 */
#ifndef strucmp
#undef strucmp
#endif
#define strucmp(lstr,rstr) struncmp(lstr,rstr,32767)


#ifdef NO_STRERROR
/*
 * replacement strerror() for systems that don't have it
 */
char *strerror(e)
int e; {
	static char buf[32];

	sprintf(buf,"errno = %d",e);
	return(buf);
	}
#endif


void strip_trailing_whitespace(buf)
char buf[]; {
	while(isspace(buf[strlen(buf)-1]))
		buf[strlen(buf)-1]=0;
	}


/*
 * for performance optimization, netproc loads the list of room names (and
 * their corresponding directory names, if applicable) into a table in memory.
 */
int load_roomnames() {
	FILE *fp;
	struct quickroom qbuf;
	int i;

	fp=fopen("./quickroom","r");
	if (fp==NULL) return(1);
	for (i=0; i<MAXROOMS; ++i) {
		if (fread((char *)&qbuf,sizeof(struct quickroom),1,fp)!=1)
			return(1);
		strcpy(roomnames[i],qbuf.QRname);
		if (qbuf.QRflags & QR_DIRECTORY)
			strcpy(roomdirs[i],qbuf.QRdirname);
		else
			strcpy(roomdirs[i],config.c_bucket_dir);
		}
	fclose(fp);
	return(0);
	}

/*
 * we also load the network/mail.sysinfo table into memory, make changes
 * as we learn more about the network from incoming messages, and write
 * the table back to disk when we're done.
 */
int load_syslist() {
	FILE *fp;
	struct syslist *stemp;
	char insys = 0;
	char buf[128];

	fp=fopen("network/mail.sysinfo","r");
	if (fp==NULL) return(1);

	while(1) {
		if (fgets(buf,128,fp)==NULL) {
			fclose(fp);
			return(0);
			}
		buf[strlen(buf)-1] = 0;
		while (isspace(buf[0])) strcpy(buf,&buf[1]);
		if (buf[0]=='#') buf[0]=0;
		if ( (insys==0) && (strlen(buf)!=0) ) {
			insys = 1;
			stemp =(struct syslist *)malloc(sizeof(struct syslist));
			stemp->next = slist;
			slist = stemp;
			strcpy(slist->s_name,buf);
			strcpy(slist->s_type,"bin");
			strcpy(slist->s_nexthop,"Mail");
			slist->s_lastcontact = 0L;
			strcpy(slist->s_humannode,"");
			strcpy(slist->s_phonenum,"");
			strcpy(slist->s_gdom,"");
			}
		else if ( (insys==1) && (strlen(buf)==0) ) {
			insys = 0;
			}
		else if ( (insys==1) && (!strncmp(buf,"bin",3)) ) {
			strcpy(slist->s_type,"bin");
			strcpy(slist->s_nexthop,&buf[4]);
			}
		else if ( (insys==1) && (!strncmp(buf,"use",3)) ) {
			strcpy(slist->s_type,"use");
			strcpy(slist->s_nexthop,&buf[4]);
			}
		else if ( (insys==1) && (!strncmp(buf,"uum",3)) ) {
			strcpy(slist->s_type,"uum");
			strcpy(slist->s_nexthop,&buf[4]);
			}
		else if ( (insys==1) && (!strncmp(buf,"lastcontact",11)) ) {
			sscanf(&buf[12],"%ld",&slist->s_lastcontact);
			}
		else if ( (insys==1) && (!strncmp(buf,"humannode",9)) ) {
			strcpy(slist->s_humannode,&buf[10]);
			}
		else if ( (insys==1) && (!strncmp(buf,"phonenum",8)) ) {
			strcpy(slist->s_phonenum,&buf[9]);
			}
		else if ( (insys==1) && (!strncmp(buf,"gdom",4)) ) {
			strcpy(slist->s_gdom,&buf[5]);
			}
		}
	}

/* now we have to set up two "special" nodes on the list: one
 * for the local node, and one for an Internet gateway
 */
void setup_special_nodes() {
	struct syslist *stemp,*slocal;

	slocal = NULL;
	for (stemp=slist; stemp!=NULL; stemp=stemp->next) {
		if (!strucmp(stemp->s_name,config.c_nodename)) slocal=stemp;
		}
	if (slocal==NULL) {
		slocal =(struct syslist *)malloc(sizeof(struct syslist));
		slocal->next = slist;
		slist = slocal;
		}
	strcpy(slocal->s_name,config.c_nodename);
	strcpy(slocal->s_type,"bin");
	strcpy(slocal->s_nexthop,"Mail");
	time(&slocal->s_lastcontact);
	strcpy(slocal->s_humannode,config.c_humannode);
	strcpy(slocal->s_phonenum,config.c_phonenum);

	slocal = NULL;
	for (stemp=slist; stemp!=NULL; stemp=stemp->next) {
		if (!strucmp(stemp->s_name,"internet")) slocal=stemp;
		}
	if (slocal==NULL) {
		slocal =(struct syslist *)malloc(sizeof(struct syslist));
		slocal->next = slist;
		slist = slocal;
		}
	strcpy(slocal->s_name,"internet");
	strcpy(slocal->s_type,"uum");
	strcpy(slocal->s_nexthop,"%s");
	time(&slocal->s_lastcontact);
	strcpy(slocal->s_humannode,"Internet Gateway");
	strcpy(slocal->s_phonenum,"");
	strcpy(slocal->s_gdom,"");

	}

/*
 * here's the routine to write the table back to disk.
 */
void rewrite_syslist() {
	struct syslist *stemp;
	FILE *newfp;
	long now;

	time(&now);
	newfp=fopen("network/mail.sysinfo","w");
	for (stemp=slist; stemp!=NULL; stemp=stemp->next) {
		if (!strucmp(stemp->s_name,config.c_nodename)) {
			time(&stemp->s_lastcontact);
			strcpy(stemp->s_type,"bin");
			strcpy(stemp->s_humannode,config.c_humannode);
			strcpy(stemp->s_phonenum,config.c_phonenum);
			}
	    /* remove systems we haven't heard from in a while */
	    if ( (stemp->s_lastcontact == 0L) 
		 || (now - stemp->s_lastcontact < EXPIRY_TIME) ) {
		fprintf(newfp,"%s\n%s %s\n",
			stemp->s_name,stemp->s_type,stemp->s_nexthop);
		if (strlen(stemp->s_phonenum) > 0) 
			fprintf(newfp,"phonenum %s\n",stemp->s_phonenum);
		if (strlen(stemp->s_gdom) > 0) 
			fprintf(newfp,"gdom %s\n",stemp->s_gdom);
		if (strlen(stemp->s_humannode) > 0) 
			fprintf(newfp,"humannode %s\n",stemp->s_humannode);
		if (stemp->s_lastcontact > 0L)
			fprintf(newfp,"lastcontact %ld %s",
				stemp->s_lastcontact,
				asctime(localtime(&stemp->s_lastcontact)));
		fprintf(newfp,"\n");
		}
	    }
	fclose(newfp);
	/* now free the list */
	while (slist!=NULL) {
		stemp = slist;
		slist = slist->next;
		free(stemp);
		}
	}


/* call this function with the node name of a system and it returns a pointer
 * to its syslist structure.
 */
struct syslist *get_sys_ptr(sysname)
char *sysname; {
	static char sysnambuf[16];
	static struct syslist *sysptrbuf = NULL;
	struct syslist *stemp;

	if ( (!strcmp(sysname,sysnambuf))
		&& (sysptrbuf!=NULL) )  return(sysptrbuf);

	strcpy(sysnambuf,sysname);
	for (stemp=slist; stemp!=NULL; stemp=stemp->next) {
		if (!strcmp(sysname,stemp->s_name)) {
			sysptrbuf = stemp;
			return(stemp);
			}
		}
	sysptrbuf = NULL;
	return(NULL);
	}


/*
 * make sure only one copy of netproc runs at a time, using lock files
 */
int set_lockfile() {
	FILE *lfp;
	int ok = 1;
	int onppid;
	char buf[64];

	if (access(LOCKFILE,0)==0) {

	/* 
	 * if the /proc filesystem is available, we can further check to
	 * make sure that the process that wrote the lock file is actually
	 * running, and didn't simply terminate and not clean up after itself
	 */
#ifdef HAVE_PROC_FS
		lfp = fopen(LOCKFILE,"r");
		fscanf(lfp,"%d",&onppid);
		fclose(lfp);
		sprintf(buf,"/proc/%d/cmdline",onppid);
		if (access(buf,0)==0) ok = 0;
#else
		ok = 0;
#endif
		}

	if (ok == 0) return(1);
	lfp=fopen(LOCKFILE,"w");
	fprintf(lfp,"%d\n",getpid());
	fclose(lfp);
	return(0);
	}

void remove_lockfile() {
	unlink(LOCKFILE);
	}

/*
 * Why both cleanup() and nq_cleanup() ?  Notice the alarm() call in
 * cleanup() .  If for some reason netproc hangs waiting for the server
 * to clean up, the alarm clock goes off and the program exits anyway.
 */
void nq_cleanup(e)
int e; {
	remove_lockfile();
	exit(e);
	}

void cleanup(e)
int e; {
	alarm(30);
	signal(SIGALRM,nq_cleanup);
	/* serv_puts("QUIT"); FIX */
	nq_cleanup(e);
	}

/*
 * This is implemented as a function rather than as a macro because the
 * client-side IPC modules expect logoff() to be defined.  They call logoff()
 * when a problem connecting or staying connected to the server occurs.
 */
void logoff(e)
int e; {
	cleanup(e);
	}

/*
 * If there is a kill file in place, this function will process it.
 */
void load_filterlist() {
	FILE *fp;
	struct filterlist *fbuf;
	char sbuf[256];
	int a,p;
	fp=fopen("./network/filterlist","r");
	if (fp==NULL) return;
	while (fgets(sbuf,256,fp)!=NULL) {
		if (sbuf[0]!='#') {
			sbuf[strlen(sbuf)-1]=0;
			fbuf=(struct filterlist *)
				malloc((long)sizeof(struct filterlist));
			fbuf->next = filter;
			filter = fbuf;
			strcpy(fbuf->f_person,"*");
			strcpy(fbuf->f_room,"*");
			strcpy(fbuf->f_system,"*");
			p = (-1);
			for (a=strlen(sbuf); a>=0; --a) if (sbuf[a]==',') p=a;
			if (p>=0) {
				sbuf[p] = 0;
				strcpy(fbuf->f_person,sbuf);
				strcpy(sbuf,&sbuf[p+1]);
				}
			for (a=strlen(sbuf); a>=0; --a) if (sbuf[a]==',') p=a;
			if (p>=0) {
				sbuf[p] = 0;
				strcpy(fbuf->f_room,sbuf);
				strcpy(sbuf,&sbuf[p+1]);
				}
			strcpy(fbuf->f_system,sbuf);
			}
		}
	fclose(fp);
	}

/* returns 1 if user/message/room combination is in the kill file */
int is_banned(k_person,k_room,k_system)
char *k_person,*k_room,*k_system; {
	struct filterlist *fptr;

	for (fptr=filter; fptr!=NULL; fptr=fptr->next) if (
	 ((!strucmp(fptr->f_person,k_person))||(!strcmp(fptr->f_person,"*")))
	&&
	 ((!strucmp(fptr->f_room,k_room))||(!strcmp(fptr->f_room,"*")))
	&&
	 ((!strucmp(fptr->f_system,k_system))||(!strcmp(fptr->f_system,"*")))
	) return(1);

	return(0);
	}

int get_sysinfo_type(name)	/* determine routing from sysinfo file */
char name[]; {
	struct syslist *stemp;
GETSN:	for (stemp=slist; stemp!=NULL; stemp=stemp->next) {
	    if (!strucmp(stemp->s_name,name)) {
		if (!strucmp(stemp->s_type,"use")) {
			strcpy(name,stemp->s_nexthop);
			goto GETSN;
			}
		if (!strucmp(stemp->s_type,"bin")) {
			return(M_BINARY);
			}
		if (!strucmp(stemp->s_type,"uum")) {
			return(M_UUCP);
			}
		}
	    }
	printf("netproc: cannot find system '%s' in mail.sysinfo\n",name);
	return(-1);
	}


int wrapgetc(fp)
FILE *fp; {
	int a;
	if (ftell(fp)>=MM_FILELEN) fseek(fp,0L,0);
	a=getc(fp);
	return(a);
	}

void fpgetfield(fp,string)
FILE *fp;
char string[]; {
	int a,b;

	strcpy(string,"");
	a=0;
	do {
		b=wrapgetc(fp);
		if (b<1) {
			string[a]=0;
			return;
			}
		string[a]=b;
		++a;
		} while (b!=0);
	}



/*
 * Load all of the fields of a message, except the actual text, into a
 * table in memory (so we know how to process the message).
 */
void msgfind(msgfile,buffer)
char *msgfile;
struct minfo *buffer; {
	int b,e,mtype,aflag;
	char bbb[1024];
	char userid[1024];
	FILE *fp;
		
	strcpy(userid,"");
	fp=fopen(msgfile,"rb");
	if (fp==NULL) {
		fprintf(stderr,"Can't open message file: %s\n",strerror(errno));
		return;
		}
	e=getc(fp);
	if (e!=255) {
		fprintf(stdout,"Incorrect message format\n");
		goto END;
		}
	mtype=getc(fp); aflag=getc(fp);
	buffer->I=0L;
	buffer->R[0]=0;
	buffer->H[0]=0;
	buffer->S[0]=0;
	buffer->B[0]=0;
	buffer->G[0]=0;

BONFGM:	b=getc(fp); if (b<0) goto END;
	if (b=='M') goto END;
	fpgetfield(fp,bbb);
	while ((bbb[0]==' ')&&(strlen(bbb)>1)) strcpy(bbb,&bbb[1]);
	if (b=='A') {
		strcpy(buffer->A,bbb);
		if (strlen(userid)==0) {
			strcpy(userid,bbb);
			for (e=0; e<strlen(userid); ++e)
				if (userid[e]==' ') userid[e]='_';
			}
		}
	if (b=='O') strcpy(buffer->O,bbb);
	if (b=='C') strcpy(buffer->C,bbb);
	if (b=='N') strcpy(buffer->N,bbb);
	if (b=='S') strcpy(buffer->S,bbb);
	if (b=='P') {
		/* extract the user id from the path */
		for (e=0; e<strlen(bbb); ++e)
			if (bbb[e]=='!') strcpy(userid,&bbb[e+1]);

		/* now find the next hop */
		for (e=0; e<strlen(bbb); ++e) if (bbb[e]=='!') bbb[e]=0;
		strcpy(buffer->nexthop,bbb);
		}
	if (b=='R') {
		for (e=0; e<strlen(bbb); ++e) if (bbb[e]=='_') bbb[e]=' ';
		strcpy(buffer->R,bbb);
		}
	if (b=='D') strcpy(buffer->D,bbb);
	if (b=='T') buffer->T=atol(bbb);
	if (b=='I') buffer->I=atol(bbb);
	if (b=='H') strcpy(buffer->H,bbb);
	if (b=='B') strcpy(buffer->B,bbb);
	if (b=='G') strcpy(buffer->G,bbb);
	goto BONFGM;

END:	if (buffer->I==0L) buffer->I=buffer->T;
	fclose(fp);
	}

void send_message(filename,retbuf)	/* send a message to the master file */
char filename[];		/* tempfilename of proper message */
struct smreturn *retbuf;	/* return information */
{

int file,a,c,d;
long bb,cc,templen,hibytes,origpos,bytes_written;
char sbuf[BUFSIZ];
char mid[30];
FILE *fp,*fp2;
struct msgmain msgmain;

	/* lock_msgmain(); */
	file=open("MMstructure",O_RDWR);
	lseek(file,0L,0);
	a=read(file,&msgmain,sizeof(struct msgmain));
	lseek(file,0L,0);
	close(file);

	origpos=msgmain.MMcurpos;
	++msgmain.MMhighest;
	sprintf(mid,"#%ld",msgmain.MMhighest);

	/* measure the message */
	fp=fopen(filename,"r");
	fseek(fp,0L,2);
	templen=ftell(fp);
	fclose(fp);
	templen=templen+((long)strlen(mid))+2L;

	/* check for FF bytes, to determine how many msgs we're overwriting */
	hibytes=0L;
	fp=fopen("msgmain","rb");
	fseek(fp,msgmain.MMcurpos,0);
	cc=ftell(fp);
	for (bb=0L; bb<templen; ++bb) {
		if (cc>=MM_FILELEN) {
			fseek(fp,0L,0);
			cc=0L;
			}
		c=getc(fp); ++cc;
		if (c>127) ++hibytes;	/* bump count if hi bit set */
		}
	fclose(fp);
	msgmain.MMlowest=msgmain.MMlowest+hibytes;

	fp=fopen("msgmain","r+");
	fseek(fp,msgmain.MMcurpos,0);
	cc=ftell(fp);
	fp2=fopen(filename,"r");
	fread((char *)sbuf,3,1,fp2);
	fwrite((char *)sbuf,3,1,fp);
	fprintf(fp,"%s",mid);
	putc(0,fp);

	/* 3 header + mid (I field) + 1 zero-byte */
	bytes_written = 4L + ((long)strlen(mid));
	/* bytes_written = 3L; */

	while (bytes_written < templen) {
		d=getc(fp2);
		if (d<0) d=0;
		putc(d,fp);
		++cc;
		if (cc>=MM_FILELEN) {
			fseek(fp,0L,0);
			cc = 0L;
			}
		++bytes_written;
		}

	msgmain.MMcurpos=ftell(fp);
	fclose(fp2);	/* done with temp file */
	fclose(fp);	/* done with master file */

	/* now update the message structure */
	file=open("MMstructure",O_RDWR);
	a=write(file,&msgmain,sizeof(struct msgmain));
	close(file);

	/* unlock_msgmain(); */

	retbuf->smnumber=msgmain.MMhighest;
	retbuf->smpos=origpos;

	}


void ship_to(filenm,sysnm)	/* send spool file filenm to system sysnm */
char *filenm;
char *sysnm; {
	char sysflnm[100];
	char commbuf1[100];
	char commbuf2[100];
	FILE *sysflfd;

#ifdef DEBUG
	fprintf(stdout,"netproc: shipping %s to %s\n",filenm,sysnm);
#endif
	sprintf(sysflnm,"./network/systems/%s",sysnm);
	sysflfd=fopen(sysflnm,"r");
	if (sysflfd==NULL) fprintf(stdout,"netproc: cannot open %s\n",sysflnm);
	fgets(commbuf1,99,sysflfd);
	commbuf1[strlen(commbuf1)-1] = 0;
	fclose(sysflfd);
	sprintf(commbuf2,commbuf1,filenm);
	system(commbuf2);
	}

/*
 * proc_file_transfer()  -  handle a simple file transfer packet
 */
void proc_file_transfer(tname)
char *tname; {	/* name of temp file containing the whole message */
	char buf[128];
	char dest_dir[32];
	FILE *tfp,*uud;
	int a,b;

	printf("netproc: processing network file transfer...\n");
	strcpy(dest_dir,config.c_bucket_dir);

	tfp=fopen(tname,"r");
	if (tfp==NULL) printf("netproc: cannot open %s\n",tname);
	getc(tfp); getc(tfp); getc(tfp);
	do {
		a=getc(tfp);
		if (a!='M') {
			fpgetfield(tfp,buf);
			if (a=='O') for (b=0; b<MAXROOMS; ++b) {
				if (!strucmp(buf,roomnames[b]))
					strcpy(dest_dir,roomdirs[b]);
				}
			}
		} while ((a!='M')&&(a>=0));
	if (a!='M') {
		fclose(tfp);
		printf("netproc: no message text for file transfer\n");
		return;
		}

	sprintf(buf,"%s/files/%s; exec uudecode",BBSDIR,dest_dir);
	uud=(FILE *)popen(buf,"w");
	if (uud==NULL) {
		printf("netproc: cannot open uudecode pipe\n");
		fclose(tfp);
		return;
		}

	fgets(buf,128,tfp);
	buf[strlen(buf)-1] = 0;
	for (a=0; a<strlen(buf); ++a) if (buf[a]=='/') buf[a]='_';
	fprintf(uud,"%s\n",buf);
	printf("netproc: %s\n",buf);
	while(a=getc(tfp), a>0) putc(a,uud);
	fclose(tfp);
	pclose(uud);
	return;
	}


/*
 * direct a message to a user's mailbox
 */
int mail_to(user,where)
char user[];
struct smreturn *where; {
	struct usersupp usersupp;
	long uspos;
	int a;
	FILE *usfp;
	long aa;

	usfp=fopen("usersupp","r+");
	if (usfp==NULL) return(-1);
	uspos = (-1L);

	while (aa=ftell(usfp),
	      fread((char *)&usersupp,sizeof(struct usersupp),1,usfp)>0) {
		if (!struncmp(usersupp.fullname,user,strlen(user)))
			uspos=aa;
		if (!strucmp(usersupp.fullname,user)) goto ENDSEARCH;
		}

ENDSEARCH: if (uspos<0L) {
		fclose(usfp);
		return(-1);
		}

	fseek(usfp,uspos,0);
	fread((char *)&usersupp,sizeof(struct usersupp),1,usfp);
	for (a=0; a<(MAILSLOTS-1); ++a) {
		usersupp.mailnum[a]=usersupp.mailnum[a+1];
		usersupp.mailpos[a]=usersupp.mailpos[a+1];
		}
	usersupp.mailnum[MAILSLOTS-1]=where->smnumber;
	usersupp.mailpos[MAILSLOTS-1]=where->smpos;
	fseek(usfp,uspos,0);
	fwrite((char *)&usersupp,sizeof(struct usersupp),1,usfp);
	fclose(usfp);
	return(0);
	}


/* send a bounce message */
void bounce(bminfo)
struct minfo *bminfo; {

	FILE *bounce;
	char bfilename[64];
	static int bseq = 1;
	long now;

	sprintf(bfilename,"./network/spoolin/bounce.%d.%d",getpid(),bseq++);
	bounce = fopen(bfilename,"wb");
	time(&now);
		
	fprintf(bounce,"%c%c%c",0xFF,MES_NORMAL,0);
	fprintf(bounce,"Ppostmaster%c",0);
	fprintf(bounce,"T%ld%c",now,0);
	fprintf(bounce,"APostmaster%c",0);
	fprintf(bounce,"OMail%c",0);
	fprintf(bounce,"N%s%c",config.c_nodename,0);
	fprintf(bounce,"H%s%c",config.c_humannode,0);
	fprintf(bounce,"R%s%c",bminfo->A,0);
	fprintf(bounce,"D%s%c",bminfo->N,0);
	fprintf(bounce,"M%s could not deliver your mail to:\n",
		config.c_humannode);
	fprintf(bounce," \n %s\n \n",bminfo->R);
	fprintf(bounce," because there is no such user on this system.\n");
	fprintf(bounce," (Unsent message does *not* follow.  ");
	fprintf(bounce,"Help to conserve bandwidth.)\n%c",0);
	fclose(bounce);
	}



void file_message(room,where)
char room[];
struct smreturn *where; {
	struct quickroom quickroom;
	struct fullroom fullroom;
	int rmnum,a,file;
	char frname[64];

	strcpy(frname,room);
	frname[sizeof(quickroom.QRname)-1]=0;
	rmnum=2;
	file=open("quickroom",O_RDWR);
	if (file<0) fprintf(stdout,"netproc: cannot open quickroom!\n");
	for (a=0; a<MAXROOMS; ++a) {
		if (read(file,&quickroom,sizeof(struct quickroom))<1)
			fprintf(stdout,"netproc: cannot read quickroom!\n");
		if (!strucmp(quickroom.QRname,frname)) rmnum=a;
		}
	lseek(file,(long)(rmnum*sizeof(struct quickroom)),0);
	if (read(file,&quickroom,sizeof(struct quickroom))<1)
		fprintf(stdout,"netproc: cannot read quickroom!\n");
	quickroom.QRhighest=where->smnumber;
	lseek(file,(long)(rmnum*sizeof(struct quickroom)),0);
	if (write(file,&quickroom,sizeof(struct quickroom))<1)
		fprintf(stdout,"netproc: cannot write to quickroom!\n");
	close(file);
	sprintf(frname,"%s/rooms/fullrm%d",BBSDIR,rmnum);
	file=open(frname,O_RDWR);
	read(file,&fullroom,sizeof(struct fullroom));
	for (a=0; a<(MSGSPERRM-1); ++a) {
		fullroom.FRnum[a]=fullroom.FRnum[a+1];
		fullroom.FRpos[a]=fullroom.FRpos[a+1];
		}
	fullroom.FRnum[MSGSPERRM-1]=where->smnumber;
	fullroom.FRpos[MSGSPERRM-1]=where->smpos;
	lseek(file,0L,0);
	write(file,&fullroom,sizeof(struct fullroom));
	close(file);
	}

/*
 * process incoming files in ./network/spoolin
 */
void inprocess() {
	FILE *fp,*message,*testfp,*ls;
	static struct minfo minfo;
	struct recentmsg recentmsg;
	struct smreturn smreturn;
	char tname[128],aaa[1024],iname[256],sfilename[256],pfilename[256];
	int a,b;
	struct syslist *stemp;
	char *ptr = NULL;

	sprintf(tname,"/tmp/net.t%d",getpid());	/* temp file name */
	sprintf(iname,"/tmp/net.i%d",getpid());	/* temp file name */

	load_filterlist();

	chdir(BBSDIR);
	
	/* Let the shell do the dirty work. Get all data from spoolin */
    do {
	sprintf(aaa,"cd %s/network/spoolin; ls",BBSDIR);
	ls=popen(aaa,"r");
	if (ls==NULL) {
		fprintf(stderr,"netproc: could not open dir cmd: %s\n",
			strerror(errno));
		}
	if (ls!=NULL) {
		do {
			ptr=fgets(sfilename,32,ls);
			if (ptr!=NULL) sfilename[strlen(sfilename)-1] = 0;
			} while( (ptr!=NULL)&&((!strcmp(sfilename,"."))
				 ||(!strcmp(sfilename,".."))));
		if (ptr!=NULL) printf("netproc: processing %s\n",sfilename);
		pclose(ls);
		}

      if (ptr!=NULL) {
	sprintf(pfilename,"%s/network/spoolin/%s",BBSDIR,sfilename);
	fprintf(stderr,"netproc: processing <%s>\n", pfilename);
	fflush(stdout);
	
	fp=fopen(pfilename,"r");
	if(fp == NULL) {
	    fprintf(stderr, "netproc: cannot open <%s>: %s\n",
			pfilename,strerror(errno));
	    fflush(stderr);
	    fp = fopen("/dev/null","r");
	    }
		
NXMSG:	/* Seek to the beginning of the next message */
	do {
		a=getc(fp);
		} while((a!=255)&&(a>=0));
	if (a<0) goto ENDSTR;
	message=fopen(tname,"w");
	putc(255,message);
	do {
		do {
			a=getc(fp);
			putc(a,message);
			} while(a!=0);
		a=getc(fp);
		putc(a,message);
		} while (a!='M');
	do {
		a=getc(fp);
		putc(a,message);
		} while(a>0);
	fclose(message);

	/* process the individual mesage */
	minfo.D[0]=0;
	minfo.C[0]=0;
	minfo.B[0]=0;
	minfo.G[0]=0;
	minfo.R[0]=0;
	msgfind(tname,&minfo);
	strncpy(recentmsg.RMnodename,minfo.N,9);
	recentmsg.RMnodename[9]=0;
	recentmsg.RMnum=minfo.I;
	printf("netproc: #%ld fm <%s> in <%s> @ <%s>\n",
		minfo.I,minfo.A,minfo.O,minfo.N);
	if (strlen(minfo.R)>0) {
		printf("         to <%s>",minfo.R);
		if (strlen(minfo.D)>0) {
			printf(" @ <%s>",minfo.D);
			}
		printf("\n");
		}
	fflush(stdout);
	if (!strucmp(minfo.D,FQDN)) strcpy(minfo.D,NODENAME);

	/* this routine updates our info on the system that sent the message */
	stemp = get_sys_ptr(minfo.N);
	if ((stemp == NULL) && (get_sys_ptr(minfo.nexthop) != NULL)) {
		/* add non-neighbor system to map */
		stemp = (struct syslist *)malloc((long)sizeof(struct syslist));
		stemp->next = slist;
		slist = stemp;
		strcpy(slist->s_name,minfo.N);
		strcpy(slist->s_type,"use");
		strcpy(slist->s_nexthop,minfo.nexthop);
		time(&slist->s_lastcontact);
		}
	else if ((stemp == NULL) && (!strucmp(minfo.N,minfo.nexthop))) {
		/* add neighbor system to map */
		sprintf(aaa,"%s/network/systems/%s",BBSDIR,minfo.N);
		testfp=fopen(aaa,"r");
		if (testfp!=NULL) {
			fclose(testfp);
			stemp = (struct syslist *)
				malloc((long)sizeof(struct syslist));
			stemp->next = slist;
			slist = stemp;
			strcpy(slist->s_name,minfo.N);
			strcpy(slist->s_type,"bin");
			strcpy(slist->s_nexthop,"Mail");
			time(&slist->s_lastcontact);
			}
		}
	/* now update last contact and long node name if we can */
	if (stemp!=NULL) {
		time(&stemp->s_lastcontact);
		if (strlen(minfo.H) > 0) strcpy(stemp->s_humannode,minfo.H);
		if (strlen(minfo.B) > 0) strcpy(stemp->s_phonenum,minfo.B);
		if (strlen(minfo.G) > 0) strcpy(stemp->s_gdom,minfo.G);
		}

	/* route the message if necessary */
	if ((strucmp(minfo.D,NODENAME))&&(minfo.D[0]!=0)) { 
		a = get_sysinfo_type(minfo.D);
		printf("netproc: routing message to system <%s>\n",minfo.D);
		if (a==M_UUCP) {
			if (fork()==0) {
				execlp("./netmailer","netmailer",
					tname,NULL);
				printf("netproc: error running netmailer: %s\n",
					strerror(errno));
				exit(errno);
				}
			else while (wait(&b)!=(-1));
			}
		else ship_to(tname,minfo.D);
		}

	/* check to see if it's a file transfer */
	else if (!struncmp(minfo.S,"FILE",4)) {
		proc_file_transfer(tname);
		}

	/* otherwise process it as a normal message */
	else {
		lock_msgmain();
		send_message(tname,&smreturn);
		if (strlen(minfo.C)==0) strcpy(minfo.C,minfo.O);
		if (is_banned(minfo.A,minfo.C,minfo.N)) {
			strcpy(minfo.C,FILTERROOM);
			printf("netproc: (message filtered)\n");
			}
		if (minfo.R[0]!=0) {
			a=mail_to(minfo.R,&smreturn);
			/* if (a<0) file_message(TWITROOM,&smreturn); */
			if (a<0) bounce(&minfo);
			}
		else {
			file_message(minfo.C,&smreturn);
			}
		unlock_msgmain();
		}

	unlink(tname);
	goto NXMSG;

ENDSTR:	fclose(fp);
	unlink(pfilename);
	}
      } while(ptr!=NULL);
    unlink(iname);
    }


int checkpath(path,sys)	/* Checks to see whether its ok to send */
char path[];		/* Returns 1 for ok, send message	*/
char sys[]; {		/* Returns 0 if message already there	*/
	int a;
	char sys2[512];
	strcpy(sys2,sys);
	strcat(sys2,"!");

#ifdef DEBUG
	printf("netproc: checkpath <%s> <%s> ... ",path,sys);
#endif
	for (a=0; a<strlen(path); ++a) {
		if (!strncmp(&path[a],sys2,strlen(sys2))) return(0);
		}
	return(1);
	}

/*
 * implement split horizon algorithm
 */
int ismsgok(mpos,mmfp,sysname)
long mpos;
FILE *mmfp;
char *sysname; {
	int a;
	int ok = 0;		/* fail safe - no path, don't send it */
	char fbuf[256];

	fseek(mmfp,mpos,0);
	if (wrapgetc(mmfp)!=255) return(0);
	wrapgetc(mmfp); wrapgetc(mmfp);

	while (a=wrapgetc(mmfp),((a!='M')&&(a!=0))) {
		fpgetfield(mmfp,fbuf);
		if (a=='P') {
			ok = checkpath(fbuf,sysname);
			}
		}
#ifdef DEBUG
	printf("%s\n", ((ok)?"SEND":"(no)") );
#endif
	return(ok);
	}

int spool_out(cmlist,destfp,sysname)	/* spool list of messages to a file */
struct msglist *cmlist;			/* returns # of msgs spooled */
FILE *destfp; 
char *sysname;
{
	struct msglist *cmptr;
	FILE *mmfp;
	char fbuf[128];
	int a;
	int msgs_spooled = 0;

	mmfp=fopen("msgmain","r");
	if (mmfp==NULL) return(0);

	for (cmptr=cmlist; cmptr!=NULL; cmptr=cmptr->next) {
		if (ismsgok(cmptr->m_pos,mmfp,sysname)) {
			++msgs_spooled;
			fflush(stdout);
			fseek(mmfp,cmptr->m_pos,0);
			fread(fbuf,3,1,mmfp);
			fwrite(fbuf,3,1,destfp);
			while (a=getc(mmfp),((a!=0)&&(a!='M'))) {
				if (a!='C') putc(a,destfp);
				fpgetfield(mmfp,fbuf);
				if (a=='P') fprintf(destfp,"%s!",NODENAME);
				if (a!='C')
					fwrite(fbuf,strlen(fbuf)+1,1,destfp);
				}
			if (a=='M') {
				putc('C',destfp);
				fwrite(roomnames[cmptr->m_room],
					strlen(roomnames[cmptr->m_room])+1,
					1,destfp);
				putc('M',destfp);
				do {
					if (ftell(mmfp)>=MM_FILELEN)
						fseek(mmfp,0L,0);
					a=getc(mmfp);
					putc(a,destfp);
					} while(a>0);
				}
			}
		}

	fclose(mmfp);
	return(msgs_spooled);
	}

void outprocess(sysname) /* send new room messages to sysname */
char *sysname; {
	char sysflnm[64];
	char srmname[32];
	char shiptocmd[128];
	char lbuf[64];
	char tempflnm[64];
	struct quickroom cquickrm;
	struct fullroom cfullrm;
	struct msglist *cmlist = NULL;
	struct rmlist *crmlist = NULL;
	struct rmlist *rmptr,*rmptr2;
	struct msglist *cmptr,*cmptr2;
	FILE *sysflfp,*qrfp,*frfp,*tempflfp;
	int a,b,file,outgoing_msgs;
	struct msgmain mmstruct;

	sprintf(tempflnm,"/tmp/%s.%d",NODENAME,getpid());
	tempflfp=fopen(tempflnm,"w");
	if (tempflfp==NULL) return;

	file=open("./MMstructure",O_RDWR);
	if (file>=0) {
		read(file,&mmstruct,sizeof(struct msgmain));
		close(file);
		}
	else mmstruct.MMlowest = 0L;

/*
 * Read system file for node in question and put together room list
 */
	sprintf(sysflnm,"%s/network/systems/%s",BBSDIR,sysname);
	sysflfp=fopen(sysflnm,"r");
	if (sysflfp==NULL) return;
	fgets(shiptocmd,128,sysflfp); shiptocmd[strlen(shiptocmd)-1]=0;
	while(!feof(sysflfp)) {
		if (fgets(srmname,32,sysflfp)==NULL) break;
		srmname[strlen(srmname)-1]=0;
		fgets(lbuf,32,sysflfp);
		rmptr=(struct rmlist *)malloc(sizeof(struct rmlist));
		rmptr->next = NULL;
		strcpy(rmptr->rm_name,srmname);
		strip_trailing_whitespace(rmptr->rm_name);
		rmptr->rm_lastsent = atol(lbuf);
		if (rmptr->rm_lastsent < mmstruct.MMlowest)
			rmptr->rm_lastsent = mmstruct.MMlowest;
		if (crmlist==NULL) crmlist=rmptr;
		else if (!strucmp(rmptr->rm_name,"control")) {
			/* control has to be first in room list */
			rmptr->next = crmlist;
			crmlist = rmptr;
			}
		else {
			rmptr2=crmlist;
			while (rmptr2->next != NULL) rmptr2=rmptr2->next;
			rmptr2->next=rmptr;
			}
		}
	fclose(sysflfp);

/*
 * Assemble list of messages to be spooled
 */
	for (rmptr=crmlist; rmptr!=NULL; rmptr=rmptr->next) {

		/* FIX   (this snippet _works_.)
		sprintf(buf,"GOTO %s",rmptr->rm_name);
		serv_puts(buf);
		serv_gets(buf);
		if (buf[0]!=2) {
			printf("%s\n",&buf[4]);
			}
		*/

		qrfp=fopen("quickroom","r");
		b=(-1);
		for (a=0; a<MAXROOMS; ++a) {
			fread((char *)&cquickrm,sizeof(struct quickroom),
				1,qrfp);
			if (!strucmp(cquickrm.QRname,rmptr->rm_name)) b=a;
			if (!strucmp(cquickrm.QRname,rmptr->rm_name)) break;
			}
		fclose(qrfp);
#ifdef DEBUG
		fprintf(stdout,"netproc: room #%02d <%s>\n",
			b,rmptr->rm_name);
#endif
		if (b<0) {
			fprintf(stderr,"netproc: room '%s' does not exist\n",
				rmptr->rm_name);
			}
		else {
			sprintf(lbuf,"%s/rooms/fullrm%d",BBSDIR,b);
			frfp=fopen(lbuf,"r");
			fread((char *)&cfullrm,sizeof(struct fullroom),1,frfp);
			fclose(frfp);
			for (a=0; a<MSGSPERRM; ++a) {
				if ( cfullrm.FRnum[a] > (rmptr->rm_lastsent) ) {
					rmptr->rm_lastsent = cfullrm.FRnum[a];
					
				cmptr=(struct msglist *)
				      malloc(sizeof(struct msglist));
					cmptr->next = NULL;
					cmptr->m_num = cfullrm.FRnum[a];
					cmptr->m_pos = cfullrm.FRpos[a];
					cmptr->m_room = b;
	
					if (cmlist == NULL) cmlist = cmptr;
					else {
						cmptr2 = cmlist;
						while (cmptr2->next != NULL)
						cmptr2 = cmptr2->next;
						cmptr2->next = cmptr;
						}
					}
				}
			}
		}

	outgoing_msgs=0; cmptr2=cmlist;	/* this loop counts the messages */
	while (cmptr2!=NULL) {
		++outgoing_msgs;
		cmptr2 = cmptr2->next;
		}
	printf("netproc: %d messages to be spooled to %s\n",
		outgoing_msgs,sysname);
	fflush(stdout);

/*
 * spool out the messages, but only if there are any
 */
	fflush(stdout);
	if (outgoing_msgs!=0) outgoing_msgs=spool_out(cmlist,tempflfp,sysname);
	printf("netproc: %d messages actually spooled\n",
		outgoing_msgs);
	fflush(stdout);

/*
 * Deallocate list of spooled messages
 */
	while(cmlist!=NULL) {
		cmptr=cmlist->next;
		free(cmlist);
		cmlist=cmptr;
		}

/*
 * Rewrite system file and deallocate room list
 */
	printf("Spooling...\n");
	fflush(stdout);
	sysflfp=fopen(sysflnm,"w");
	fprintf(sysflfp,"%s\n",shiptocmd);
	for (rmptr=crmlist; rmptr!=NULL; rmptr=rmptr->next)  
		fprintf(sysflfp,"%s\n%ld\n",rmptr->rm_name,rmptr->rm_lastsent);
	fclose(sysflfp);
	while(crmlist!=NULL) {
		rmptr=crmlist->next;
		free(crmlist);
		crmlist=rmptr;
		}

/* 
 * Close temporary file, ship it out, and return
 */
	fclose(tempflfp);
	if (outgoing_msgs!=0) ship_to(tempflnm,sysname);
	unlink(tempflnm);
	}


/*
 * connect netproc to the local server
 */
void np_attach_to_server() {
	char buf[256];

	printf("Attaching to server...\n");
	attach_to_server();
	serv_gets(buf);
	printf("%s\n",&buf[4]);
	sprintf(buf,"IPGM %s",SYSPGMPW);
	serv_puts(buf);
	serv_gets(buf);
	printf("%s\n",&buf[4]);
	if (buf[0]!='2') {
		cleanup(2);
		}
	}



/*
 * main
 */
void main(argc,argv)
int argc;
char *argv[];
{
	char allst[32];
	FILE *allfp;

	freopen(NPLOGFILE,NPLOGMODE,stdout);
	freopen(NPLOGFILE,NPLOGMODE,stderr);

	get_config();

	if (set_lockfile()!=0) {
		fprintf(stderr,"netproc: lock file exists: already running\n");
		cleanup(1);
		}

	signal(SIGINT,cleanup);
	signal(SIGQUIT,cleanup);
	signal(SIGHUP,cleanup);
	signal(SIGTERM,cleanup);

	printf("netproc: started.  pid=%d\n",getpid());
	/* FIX np_attach_to_server(); */

	if (load_roomnames()!=0) fprintf(stdout,"netproc: cannot load rooms\n");
	if (load_syslist()!=0) fprintf(stdout,"netproc: cannot load sysinfo\n");
	setup_special_nodes();
	inprocess();	/* first collect incoming stuff */
	if ((argc==2) && (strcmp(argv[1],"all"))) outprocess(argv[1]);
	if ((argc==2) && (!strcmp(argv[1],"all"))) {
		allfp=(FILE *)popen("cd ./network/systems; ls","r");
		if (allfp!=NULL) {
			while (fgets(allst,32,allfp)!=NULL) {
				allst[strlen(allst)-1] = 0;
				outprocess(allst);
				}
			pclose(allfp);
			}
		}
	inprocess();	/* incoming again in case anything new was generated */
	rewrite_syslist();
	unlock_msgmain(); /* FIX do we need this? */
	printf("netproc: processing ended.\n");
	cleanup(0);
	}

